package betterwithmods.module.gameplay.breeding_harness;

import betterwithmods.BWMod;
import betterwithmods.common.BWMItems;
import betterwithmods.common.items.ItemBreedingHarness;
import betterwithmods.module.Feature;
import betterwithmods.network.MessageHarnessSync;
import betterwithmods.network.NetworkHandler;
import betterwithmods.util.InvUtils;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.SharedMonsterAttributes;
import net.minecraft.entity.passive.EntityCow;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.SoundEvents;
import net.minecraft.item.ItemStack;
import net.minecraft.util.EnumHand;
import net.minecraft.util.ResourceLocation;
import net.minecraft.util.SoundCategory;
import net.minecraft.util.SoundEvent;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.IWorldEventListener;
import net.minecraft.world.World;
import net.minecraftforge.common.capabilities.CapabilityManager;
import net.minecraftforge.event.AttachCapabilitiesEvent;
import net.minecraftforge.event.entity.living.LivingEvent;
import net.minecraftforge.event.entity.player.PlayerEvent;
import net.minecraftforge.event.entity.player.PlayerInteractEvent;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.items.ItemHandlerHelper;

import javax.annotation.Nullable;

public class BreedingHarness extends Feature {

    public BreedingHarness() {
    }

    @Override
    public void preInit(FMLPreInitializationEvent event) {
        BWMItems.registerItem(BWMItems.BREEDING_HARNESS);
        CapabilityManager.INSTANCE.register(CapabilityHarness.class, new CapabilityHarness.Storage(), CapabilityHarness::new);
    }

    private static final ResourceLocation CAPABILITY = new ResourceLocation(BWMod.MODID, "harness");

    @SubscribeEvent
    public void onAttach(AttachCapabilitiesEvent<Entity> event) {
        Entity entity = event.getObject();
        if (harnessEntity(entity)) {
            event.addCapability(CAPABILITY, new CapabilityHarness());
        }
    }

    @SubscribeEvent
    public void onTrack(PlayerEvent.StartTracking event) {
        if(event.getEntityPlayer().field_70170_p.field_72995_K)
            return;
        Entity entity = event.getTarget();
        CapabilityHarness cap = getCapability(entity);
        if(cap != null) {
            NetworkHandler.sendToAllAround(new MessageHarnessSync(entity.func_145782_y(), cap.getHarness()), entity.func_130014_f_(), entity.func_180425_c());
        }
    }

    @SubscribeEvent
    public void onEntityInteract(PlayerInteractEvent.EntityInteract event) {
        if (event.getWorld().field_72995_K)
            return;
        Entity entity = event.getTarget();

        CapabilityHarness cap = getCapability(entity);
        if (cap != null) {
            ItemStack hand = event.getItemStack();
            ItemStack harness = cap.getHarness();
            if (harness.func_190926_b() && !event.getEntityPlayer().func_70093_af()) {
                if (hand.func_77973_b() instanceof ItemBreedingHarness) {
                    cap.setHarness(InvUtils.setCount(hand.func_77946_l(), 1));
                    hand.func_190918_g(1);
                    event.getWorld().func_184133_a(null, entity.func_180425_c(), SoundEvents.field_187728_s, SoundCategory.NEUTRAL, 0.5f, 1.3f);
                    event.getWorld().func_184133_a(null, entity.func_180425_c(), SoundEvents.field_187713_n, SoundCategory.NEUTRAL, 0.5f, 1.3f);
                    event.getEntityPlayer().func_184609_a(EnumHand.MAIN_HAND);
                }
            } else if (!harness.func_190926_b() && event.getEntityPlayer().func_70093_af() && hand.func_190926_b()) {
                ItemHandlerHelper.giveItemToPlayer(event.getEntityPlayer(), harness.func_77946_l());
                harness.func_190918_g(1);
                event.getWorld().func_184133_a(null, entity.func_180425_c(), SoundEvents.field_187728_s, SoundCategory.NEUTRAL, 1, 1);
                event.getWorld().func_184133_a(null, entity.func_180425_c(), SoundEvents.field_187713_n, SoundCategory.NEUTRAL, 1, 1f);
                event.getEntityPlayer().func_184609_a(EnumHand.MAIN_HAND);
            }
            NetworkHandler.sendToAllAround(new MessageHarnessSync(entity.func_145782_y(), cap.getHarness()), event.getWorld(), event.getPos());
        }

    }


    @SubscribeEvent
    public static void onLivingTick(LivingEvent.LivingUpdateEvent e) {
        EntityLivingBase entity = e.getEntityLiving();

        CapabilityHarness cap = getCapability(entity);
        if (cap != null && cap.getHarness().func_77973_b() instanceof ItemBreedingHarness) {
            entity.func_110148_a(SharedMonsterAttributes.field_111263_d).func_111128_a(-1);
        } else {
            entity.func_110148_a(SharedMonsterAttributes.field_111263_d).func_111128_a(0.25);
        }
    }


    public static CapabilityHarness getCapability(Entity entity) {
        if (entity.hasCapability(CapabilityHarness.HARNESS_CAPABILITY, null)) {
            return entity.getCapability(CapabilityHarness.HARNESS_CAPABILITY, null);
        }
        return null;
    }

    public static boolean hasHarness(Entity entity) {
        CapabilityHarness cap = getCapability(entity);
        return cap != null && cap.getHarness().func_77973_b() instanceof ItemBreedingHarness;
    }

    public static boolean harnessEntity(Entity entity) {
        return entity instanceof EntityCow;
    }

    @Override
    public boolean hasSubscriptions() {
        return true;
    }


    public class HarnessListener implements IWorldEventListener {

        @Override
        public void func_184376_a(World worldIn, BlockPos pos, IBlockState oldState, IBlockState newState, int flags) {

        }

        @Override
        public void func_174959_b(BlockPos pos) {

        }

        @Override
        public void func_147585_a(int x1, int y1, int z1, int x2, int y2, int z2) {

        }

        @Override
        public void func_184375_a(@Nullable EntityPlayer player, SoundEvent soundIn, SoundCategory category, double x, double y, double z, float volume, float pitch) {

        }

        @Override
        public void func_184377_a(SoundEvent soundIn, BlockPos pos) {

        }

        @Override
        public void func_180442_a(int particleID, boolean ignoreRange, double xCoord, double yCoord, double zCoord, double xSpeed, double ySpeed, double zSpeed, int... parameters) {

        }

        @Override
        public void func_190570_a(int id, boolean ignoreRange, boolean p_190570_3_, double x, double y, double z, double xSpeed, double ySpeed, double zSpeed, int... parameters) {

        }

        @Override
        public void func_72703_a(Entity entity) {
            if (entity.field_70170_p == null || entity.field_70170_p.field_72995_K)
                return;
            CapabilityHarness cap = getCapability(entity);
            if (cap != null) {
                NetworkHandler.sendToAllAround(new MessageHarnessSync(entity.func_145782_y(), cap.getHarness()), entity.field_70170_p, entity.func_180425_c());
            }
        }

        @Override
        public void func_72709_b(Entity entityIn) {

        }

        @Override
        public void func_180440_a(int soundID, BlockPos pos, int data) {

        }

        @Override
        public void func_180439_a(EntityPlayer player, int type, BlockPos blockPosIn, int data) {

        }

        @Override
        public void func_180441_b(int breakerId, BlockPos pos, int progress) {

        }
    }
}
